/*****************************************************************************
** PBBACK.PRG
**
** ProVision:Backup
** CA-Clipper 5.0 surface backup sample
**
** Copyright 1993 SofDesign International, Inc.
** All rights reserved
**
** tab spacing = 3
**
*****************************************************************************/

#include "inkey.ch"
#include "pb.ch"


// direct output to stderr
#xcommand	? <exp> ;
				=> ;
				OUTERR( CRLF + <exp> )

#xcommand	?? <exp> ;
				=> ;
				OUTERR( <exp> )

#define CRLF	CHR(13) + CHR(10)			// carriage return and line feed


STATIC	lCancelled	:= .F.



/*****************************************************************************
** Main( <cFileSpec>, [ <cDrive> ] ) --> NIL
**
** This is the main module of a simple PVB program using TTY-type output.
**
** Parameters:
**		<cFileSpec>		a filespec to be backed up; may contain /S and /H
**								switches, but if so, there cannot be any spaces
**								between the filespec and the switches
**
**		<cDrive>			the destination drive (A: or B:); may contain a format
**								switch of /360, /1.2, /720, /1.44, or /2.88, but if
**								so, there cannot be any spaces between the drive
**								and the switch
**
** Returns:
**		NIL
**
** The limitation of no spaces before switches applies to this sample
** program only, because it only retrieves the first two parameters from
** the command line.  PVB itself allows the /S and /H switches to appear
** anywhere in an include filespec.
**
*****************************************************************************/
FUNCTION Main( cFileSpec, cDrive )

	LOCAL	cEstTime

	LOCAL	nEstTime


	// validate command line params
	IF ( EMPTY( cFileSpec ) )
		? "Usage:  PBBACK <filespec>[/S][/H] [<drive>[/360|/1.2|/720|/1.44|/2.88]]"
		?
		? "Backs up the specified filespec to the specified drive.  There can"
		? "be no spaces between the filespec and switches or between the drive"
		? "and format."
		?
		? "/S means include subdirectories."
		? "/H means include hidden files."
		? "<drive> defaults to A:"
		?
		RETURN( NIL )
	END IF // ( EMPTY( cFileSpec ) )

	IF ( EMPTY( cDrive ) )
		cDrive	:= "A"
	END IF // ( EMPTY( cDrive ) )

	// specify the disk type?
	IF ( "/360" $ cDrive )
		pbSetDiskType( 0, DTYPE_360 )
		pbSetDiskType( 1, DTYPE_360 )
	ELSEIF ( "/1.2" $ cDrive )
		pbSetDiskType( 0, DTYPE_1_2 )
		pbSetDiskType( 1, DTYPE_1_2 )
	ELSEIF ( "/720" $ cDrive )
		pbSetDiskType( 0, DTYPE_720 )
		pbSetDiskType( 1, DTYPE_720 )
	ELSEIF ( "/1.44" $ cDrive )
		pbSetDiskType( 0, DTYPE_1_44 )
		pbSetDiskType( 1, DTYPE_1_44 )
	ELSEIF ( "/2.88" $ cDrive )
		pbSetDiskType( 0, DTYPE_2_88 )
		pbSetDiskType( 1, DTYPE_2_88 )
	END IF // ( "/360" $ cDrive )

	// set option to format entire last disk
	pbSetFormatAll( .T. )

	// turn off verify
//	pbSetVerify( .F. )

	// do an estimate
	IF ( pbEstimate( { cFileSpec }, {}, cDrive, ;
			{ | nMessage | pbCallback( nMessage ) } ) )

		// display the estimate
		nEstTime	:= pbEstValue( EST_TIME )
		cEstTime	:= STR( INT( nEstTime / 3600 ), 2, 0 ) + ":"
		nEstTime	-= INT( nEstTime / 3600 ) * 3600
		cEstTime	+= STRZERO( INT( nEstTime / 60 ), 2 ) + ":"
		nEstTime	-= INT( nEstTime / 60 ) * 60
		cEstTime	+= STRZERO( nEstTime, 2 )
		?? CHR( 13 ) + PADR( "Estimate:  " + cEstTime, 79 )
		? STR( pbEstValue( EST_FILES ), 9, 0 ) + " file"
		IF ( pbEstValue( EST_FILES ) !=  1 )
			?? "s"
		END IF // ( pbEstValue( EST_FILES ) !=  1 )
		? STR( pbEstValue( EST_BYTES ), 9, 0 ) + " byte"
		IF ( pbEstValue( EST_BYTES ) !=  1 )
			?? "s"
		END IF // ( pbEstValue( EST_BYTES ) !=  1 )
		? STR( pbEstValue( EST_VOLS ), 9, 0 ) + " disk"
		IF ( pbEstValue( EST_VOLS ) != 1 )
			?? "s"
		END IF // ( pbEstValue( EST_VOLS ) != 1 )
		? "Compression may change these figures.  Press a key..."
		IF ( INKEY( 0 ) == K_ESC )
			RETURN( NIL )
		END IF // ( INKEY( 0 ) == K_ESC )
		?? CRLF

	ELSE

		// user cancelled the estimate
		RETURN( NIL )

	END IF // ( pbEstimate( { cFileSpec }, {}, cDrive, ...

	// perform the backup
	IF ( .NOT. pbBackup( { cFileSpec }, {}, cDrive, ;
			{ | nMessage | pbCallback( nMessage ) } ) )

		IF ( .NOT. lCancelled )
			// display error message
			? CHR( 7 ) + "Error=" + STRZERO( pbError(), 4 )
			IF ( pbError() == ERROR_DOSERROR )
				?? " (DOS error=" + STR( pbDOSError(), 3, 0 ) + ")"
			ELSEIF ( pbError() == ERROR_BIOSERROR )
				?? " (BIOS error=" + STR( pbDOSError(), 3, 0 ) + ")"
			END IF // ( pbError() == ERROR_DOSERROR )
			? pb_TransError( pbError(), pbDOSError() )
			?? CRLF
		END IF // ( .NOT. lCancelled )

	END IF // ( .NOT. pbBackup( { cFileSpec }, {}, cDrive, ...

	RETURN( NIL )
	// END Main( cFileSpec, cDrive )



/*****************************************************************************
** STATIC pbCallback( <nMessage> ) --> <nResult>
**
** This is a sample PVB callback function using TTY-type output.
**
** Parameters:
**		<nMessage>		a "message" about the status of the backup
**
** Returns:
**		pbCallback() returns a number signifying the action PVB should take.
** 	If the backup should continue with the default action, BACT_OK is
**		returned.  If the backup should be cancelled, BACT_CANCEL is returned.
**		Errors ERROR_CONTAINSDATA and ERROR_WRITE and errors dealing with
**		opening and reading the file have a special return value, BACT_REDO.
**
** With ERROR_CONTAINSDATA, a return value of BACT_REDO means to overwrite
** the disk.  A synonym is BACT_OVERWRITE.  If BACT_OK is returned, PVB
** assumes the user has inserted a new disk and wished to try again.
**
** With ERROR_WRITE, BACT_REDO means to "rewind" the backup to the beginning
** of the current disk.  A synonym is BACT_SKIPVOL.  PVB assumes the user
** has inserted a new disk.  If BACT_OK is returned, PVB retries the write
** operation.
**
** With errors dealing with opening and reading the file, a return value of
** BACT_REDO means to skip the rest of the file and continue with the next
** file.  A synonym is BACT_SKIPFILE.  If BACT_OK is returned, PVB attempts
** to retry or continue with the same file.
**
*****************************************************************************/
STATIC FUNCTION pbCallback( nMessage )

	STATIC	lMediaError	:= .F.

	LOCAL	nDiskType, ;
			nKey


	IF ( INKEY() == K_ESC )
		lCancelled	:= .T.
		RETURN( BACT_CANCEL )
	END IF // ( INKEY() == K_ESC )

	DO CASE
	CASE ( nMessage == BOP_BEGIN )

	CASE ( nMessage == BOP_CANCEL )
		? "Backup cancelled." + CRLF
		RETURN( BACT_CANCEL )

	CASE ( nMessage == BOP_COMPLETE )
		? "Backup is complete." + CRLF

	CASE ( nMessage == BOP_END )

	CASE ( nMessage == BOP_ERROR )
		IF ( INT( pbError() / 100 ) == 3 )

			// test for errors in the "300" range; these normally require
			// insertion of a new disk
			DO CASE
			CASE ( pbError() == ERROR_CONTAINSDATA )
				? "This disk contains data.  Existing data will be overwritten"
				? "if you continue.  Do you want to overwrite this disk? "
				nKey	:= 0
				WHILE ( ( nKey != K_ESC ) .AND. ;
						( UPPER( CHR( nKey ) ) != "Y" ) .AND. ;
						( UPPER( CHR( nKey ) ) != "N" ) )
					nKey	:= INKEY( 0 )
				END WHILE // ( ( nKey != K_ESC ) .AND. ...
				IF ( nKey == K_ESC )
					lCancelled	:= .T.
					RETURN( BACT_CANCEL )
				END IF // ( nKey == K_ESC )
				?? UPPER( CHR( nKey ) ) + CRLF
				IF ( UPPER( CHR( nKey ) ) == "Y" )
					RETURN( BACT_OVERWRITE )
				END IF // ( UPPER( CHR( nKey ) ) == "Y" )

			CASE ( pbError() == ERROR_DRIVENOTREADY )
				? "Drive not ready."

			CASE ( pbError() == ERROR_MEDIATYPE )
				// This code attempts to take care of FORMATTED lower-density
				// disks inserted in high-density drives.
				IF ( .NOT. lMediaError )
					nDiskType	:= pbGetDiskType( pbDriveNum() )
					DO CASE
					CASE ( nDiskType == DTYPE_1_2 )
						pbSetDiskType( pbDriveNum(), DTYPE_360 )

					CASE ( nDiskType == DTYPE_1_44 )
						pbSetDiskType( pbDriveNum(), DTYPE_720 )

					CASE ( nDiskType == DTYPE_2_88 )
						pbSetDiskType( pbDriveNum(), DTYPE_1_44 )

					OTHERWISE
						lMediaError	:= .T.

					END CASE
				END IF // ( .NOT. lMediaError )

				IF ( lMediaError )
					// If the media type error still occurs after trying all
					// the possible densities for this drive, display a message.
					? "This disk does not match the media type specified for the"
					? "backup.  You must now insert a new disk."
				ELSE
					pb_Reinit( pbDriveNum() )
					RETURN( BACT_OK )
				END IF // ( lMediaError )

			CASE ( pbError() == ERROR_SAMESET )
				? "This is disk " + LTRIM( STR( pbVolNum(), 3, 0 ) ) + ;
					" from the same backup set."

			CASE ( pbError() == ERROR_SYSTEMAREA )
				? "The system area of this disk is not usable.  This indicates"
				? "that the disk is bad or is the wrong media type.  You must"
				? "now insert a new disk."

			CASE ( pbError() == ERROR_TIMEOUT )
				? "Timed out waiting for a disk."

			CASE ( pbError() == ERROR_WRITE )
				? "A bad sector was encountered while writing to this disk."
				? "The disk is not usable by ProVision:Backup.  Do you want to"
				? "[R]etry or [I]nsert a new disk? "
				nKey	:= 0
				WHILE ( ( nKey != K_ESC ) .AND. ;
						( UPPER( CHR( nKey ) ) != "R" ) .AND. ;
						( UPPER( CHR( nKey ) ) != "I" ) )
					nKey	:= INKEY( 0 )
				END WHILE // ( ( nKey != K_ESC ) .AND. ...
				IF ( nKey == K_ESC )
					lCancelled	:= .T.
					RETURN( BACT_CANCEL )
				END IF // ( nKey == K_ESC )
				?? UPPER( CHR( nKey ) ) + CRLF
				IF ( UPPER( CHR( nKey ) ) == "R" )
					RETURN( BACT_OK )
				END IF // ( UPPER( CHR( nKey ) ) == "R" )

			CASE ( pbError() == ERROR_WRITEPROTECT )
				? "This disk is write-protected."

			END CASE

			? "Insert disk " + LTRIM( STR( pbWantVolNum(), 3, 0 ) ) + ;
				" in drive " + CHR( pbDriveNum() + 65 ) + ;
				":, and press a key..." + CRLF
			IF ( INKEY( 0 ) == K_ESC )
				lCancelled	:= .T.
				RETURN( BACT_CANCEL )
			ELSEIF ( pbError() == ERROR_WRITE )
				RETURN( BACT_SKIPVOL )
			END IF // ( INKEY( 0 ) == K_ESC )

		ELSE

			DO CASE
			CASE ( pbError() == ERROR_FILENOTFOUND )
				? "File not found (" + pbFilename() + ")." + CRLF

			CASE ( pbError() == ERROR_FILEOPEN )
				? "Error opening " + pbFilename() + "." + CRLF

			CASE ( pbError() == ERROR_READ )
				? "Error reading " + pbFilename() + "." + CRLF

			OTHERWISE
				? pb_TransError( pbError(), pbDOSError() ) + "." + CRLF

			END CASE

			? "Do you want to skip to the next file? "
			nKey	:= 0
			WHILE ( ( nKey != K_ESC ) .AND. ;
					( UPPER( CHR( nKey ) ) != "Y" ) .AND. ;
					( UPPER( CHR( nKey ) ) != "N" ) )
				nKey	:= INKEY( 0 )
			END WHILE // ( ( nKey != K_ESC ) .AND. ...
			IF ( nKey == K_ESC )
				lCancelled	:= .T.
				RETURN( BACT_CANCEL )
			END IF // ( nKey == K_ESC )
			?? UPPER( CHR( nKey ) ) + CRLF
			IF ( UPPER( CHR( nKey ) ) == "Y" )
				RETURN( BACT_SKIPFILE )
			END IF // ( UPPER( CHR( nKey ) ) == "Y" )

		END IF // ( INT( pbError() / 100 ) == 3 )

	CASE ( nMessage == BOP_FILEBEGIN )
		?? CHR( 13 ) + PADR( pbFilename() + " (CTYPE=" + ;
			STR( pbCmpType(), 1, 0 ) + ")", 79 ) + CRLF

	CASE ( nMessage == BOP_FILEEND )

	CASE ( nMessage == BOP_FILEESTIMATE )
		?? CHR( 13 ) + PADR( "Estimating " + pbFilename(), 79 )

	CASE ( nMessage == BOP_NEXTVOL )
		? "Insert disk " + LTRIM( STR( pbWantVolNum(), 2, 0 ) ) + ;
			" in drive " + CHR( pbDriveNum() + 65 ) + ":"
		IF ( pbGetDetectChg() )
			?? "..." + CRLF
		ELSE
			?? ", and press a key..." + CRLF
			IF ( INKEY( 0 ) == K_ESC )
				lCancelled	:= .T.
				RETURN( BACT_CANCEL )
			END IF // ( INKEY( 0 ) == K_ESC )
		END IF // ( pbGetDetectChg() )

	CASE ( nMessage == BOP_RESETARCHIVE )
		?? "Resetting archive bits..."

	CASE ( nMessage == BOP_TRACKFORMATTED )
		?? CHR( 13 ) + "Formatting and writing cylinder " + ;
			STRZERO( pbCylNum(), 2 ) + " head " + STRZERO( pbHeadNum(), 2 )
		IF ( .NOT. EMPTY( pbFilename() ) )
			?? "  " + STR( ( pbFileRead() * 100 ) / pbFileSize(), 3, 0 ) + "%"
		END IF // ( .NOT. EMPTY( pbFilename() ) )

	CASE ( nMessage == BOP_TRACKWRITTEN )
		?? CHR( 13 ) + "Writing cylinder " + STRZERO( pbCylNum(), 2 ) + ;
			" head " + STRZERO( pbHeadNum(), 2 ) + "  " + ;
			STR( ( pbFileRead() * 100 ) / pbFileSize(), 3, 0 ) + "%"

	END CASE

	RETURN( BACT_OK )
	// END pbCallback( nMessage )

