******************************************************************************
** PBBACK2.PRG
**
** ProVision:Backup
** CA-Clipper 5.0 surface interactive backup sample
**
** Copyright 1993 SofDesign International, Inc.
** All rights reserved
**
** tab spacing = 3
**
******************************************************************************

#include "inkey.ch"
#include "pb.ch"



/*****************************************************************************
** Main() --> NIL
**
** This is the main module of a simple interactive PVB program.
**
** Returns:
**		NIL
**
*****************************************************************************/
FUNCTION Main()

	LOCAL GetList := {}

	// GET variables
	LOCAL acInclude[3]
	LOCAL alSubDir[3]
	LOCAL acExclude[3]
	LOCAL cSetName, lEncrypt, cPassword, lVerify, lTruename, lAlwaysFormat
	LOCAL lAutoDetect, nCompType, cDrive, nDriveType, nOverwrite

	// Other variables
	LOCAL i, nEstTime, cEstTime, cColor


	// Clear the screen and paint the input screen
	IF ISCOLOR()
		SET COLOR TO +W/B, B/W
	ELSE
		SET COLOR TO W/N, N/W
	ENDIF

	CLEAR SCREEN
	@ 01,07 SAY "ProVision:Backup (Sample Backup) by SofDesign International, Inc."
	@ 03,13 SAY ""+replicate('',20)+""+replicate('',8)+" "+replicate('',20)+""
	@ 04,13 SAY "Include Filespecs:  SubDirs? Exclude Filespecs:  "
	@ 05,13 SAY ""+replicate('',20)+""+replicate('',8)+" "+replicate('',20)+""
	@ 06,13 SAY ""+space(20)+""+space(8)+" "+space(20)+""
	@ 07,13 SAY ""+space(20)+""+space(8)+" "+space(20)+""
	@ 08,13 SAY ""+space(20)+""+space(8)+" "+space(20)+""
	@ 09,13 SAY ""+replicate('',20)+""+replicate('',8)+" "+replicate('',20)+""
	@ 10,02 SAY ""+replicate('',36)+""+replicate('',37)+""
	@ 11,02 SAY ""+space(10)+"Backup set name:"+space(10)+"     Backup to drive (A/B):"+space(10)+""
	@ 12,02 SAY ""+space(17)+"Encrypt?:"+space(10)+"     Backup drive capacity:"+space(10)+""
	@ 13,02 SAY ""+space(17)+"Password:"+space(10)+""+space(7)+"1) 360K,  2) 1.2M"+space(13)+""
	@ 14,02 SAY ""+space(6)+"Verify Disk Writes?:"+space(10)+""+space(7)+"3) 720K,  4) 1.44M"+space(12)+""
	@ 15,02 SAY " Fully qualify filenames?:"+space(10)+""+space(7)+"5) 2.88M  6) Auto"+space(13)+""
	@ 16,02 SAY " Always format diskettes?:"+space(10)+"     Overwrite Warnings...:"+space(10)+""
	@ 17,02 SAY "  Detect diskette change?:"+space(10)+""+space(7)+"1) Never warn"+space(17)+""
	@ 18,02 SAY ""+space(9)+"Compression Type:"+space(10)+""+space(7)+"2) Non-PVB disks"+space(14)+""
	@ 19,02 SAY ""+space(11)+"1) None"+space(18)+""+space(7)+"3) Always warn"+space(16)+""
	@ 20,02 SAY ""+space(11)+"2) Save Time"+space(13)+""+space(37)+""
	@ 21,02 SAY ""+space(11)+"3) Save Disks"+space(12)+""+space(37)+""
	@ 22,02 SAY ""+replicate('',36)+""+replicate('',37)+""

	// Initialize the input variables
	AFILL(acInclude, SPACE(20))
	AFILL(alSubDir, .F.)
	AFILL(acExclude, SPACE(20))
	cSetName			:= DTOS(DATE())
	lEncrypt			:= .F.
	cPassword		:= SPACE(14)
	lVerify			:= .F.
	lTruename		:= .T.
	lAlwaysFormat	:= .F.
	lAutoDetect		:= .T.
	nCompType		:= 3
	cDrive			:= "A"
	nDriveType		:= 6
	nOverwrite		:= 2

	// Do the GETs
	@ 06,14 GET acInclude[1]  PICTURE "@!"
	@ 06,38 GET alSubDir[1]   PICTURE "Y"
	@ 07,14 GET acInclude[2]  PICTURE "@!"
	@ 07,38 GET alSubDir[2]   PICTURE "Y"
	@ 08,14 GET acInclude[3]  PICTURE "@!"
	@ 08,38 GET alSubDir[3]   PICTURE "Y"
	@ 06,46 GET acExclude[1]  PICTURE "@!"
	@ 07,46 GET acExclude[2]  PICTURE "@!"
	@ 08,46 GET acExclude[3]  PICTURE "@!"
	@ 11,30 GET cSetName      PICTURE "@!" VALID !EMPTY(cSetName)
	@ 12,30 GET lEncrypt      PICTURE "Y"
	@ 13,30 GET cPassword     PICTURE "@!S8"
	@ 14,30 GET lVerify       PICTURE "Y"
	@ 15,30 GET lTruename     PICTURE "Y"
	@ 16,30 GET lAlwaysFormat PICTURE "Y"
	@ 17,30 GET lAutoDetect   PICTURE "Y"
	@ 18,30 GET nCompType     PICTURE "9" RANGE 1,3
	@ 11,68 GET cDrive        PICTURE "!" VALID cDrive $ "AB"
	@ 12,68 GET nDriveType    PICTURE "9" RANGE 1,6
	@ 16,68 GET nOverwrite    PICTURE "9" RANGE 1,3

	// Get input and return if escape is pressed
	READ
	IF LASTKEY() == K_ESC
		RETURN(NIL)
	ENDIF

	// Test for include subdirs
	FOR i := 1 TO 3
		acInclude[i] := LTRIM( RTRIM( acInclude[i] ) ) + ;
			IIF( alSubDir[i], "/S", "" )
		acExclude[i] := LTRIM( RTRIM( acExclude[i] ) )
	NEXT i 

	// Set the various options
	pbSetSetName(ALLTRIM(cSetName))
	pbSetEncrypt(IIF(lEncrypt, ETYPE_ENCODE, ETYPE_NONE))
	pbSetPassword(ALLTRIM(cPassword))
	pbSetVerify(lVerify)
	pbSetUseTrueName(lTruename)
	pbSetAlwaysFmt(lAlwaysFormat)
	pbSetDetectChg(lAutoDetect)
	pbSetCmpStrat(nCompType - 1)
	pbSetDiskType(ASC(cDrive) - ASC("A"), nDriveType - 1 )
	pbSetOverwriteOpt(nOverwrite - 1)

	// Create a psuedo-window
	IF ISCOLOR()
		SET COLOR TO +W/R
	ELSE
		SET COLOR TO N/W
	ENDIF
	@ 04,09 TO 21,70
	@ 05,10 CLEAR TO 20,69
	@ 20,10 SAY ""


	// Do an estimate
	IF pbEstimate(acInclude, acExclude, cDrive, ;
			{|nMessage| pbCallback(nMessage)})

		// Display the estimate
		nEstTime	:= pbEstValue(EST_TIME)
		cEstTime := STR(INT(nEstTime / 3600), 2, 0) + ":"
		nEstTime := nEstTime - INT(nEstTime / 3600) * 3600
		cEstTime := cEstTime + STRZERO(INT(nEstTime / 60), 2) + ":"
		nEstTime := nEstTime - INT(nEstTime / 60) * 60
		cEstTime := cEstTime + STRZERO(nEstTime, 2)
		PrintLine("Estimate:  " + cEstTime)
		PrintLine(STR(pbEstValue(EST_FILES), 9, 0) + " file(s)")
		PrintLine(STR(pbEstValue(EST_BYTES), 9, 0) + " byte(s)")
		PrintLine(STR(pbEstValue(EST_VOLS), 9, 0) + " disk(s)")
		PrintLine("Compression may change these figures.  Press a key...")
		IF INKEY(0) == K_ESC
			RETURN(NIL)
		ENDIF
		PrintLine("")

	ENDIF


	// Perform the backup
	IF !pbBackup(acInclude, acExclude, cDrive, ;
			{|nMessage| pbCallback(nMessage)})

		// Display error message
		IF pbError() != ERROR_NONE
			TONE(100, 3)
			PrintLine("Error=" + STRZERO(pbError(), 4))
			IF pbError() == 205
				PrintLine(" (DOS error=" + STR(pbDOSError(), 3, 0) + ")")
			ELSEIF pbError() == 206
				PrintLine(" (BIOS error=" + STR(pbDOSError(), 3, 0) + ")")
			ENDIF
			PrintLine("")
		ENDIF

	ENDIF


	// Clear status line at bottom
	cColor := SETCOLOR()
	IF ISCOLOR()
		SET COLOR TO +W/B, B/W
	ELSE
		SET COLOR TO W/N, N/W
	ENDIF
	@ 24,00 CLEAR TO 24,79
	SETCOLOR(cColor)

	// Ask user to press a key
	PrintLine("Press any key to return to DOS...")

	// Wait for keystroke
	CLEAR TYPEAHEAD
	INKEY(0)

	// Clear screen and return
	SET COLOR TO W/N
	CLEAR SCREEN

	RETURN(NIL)



******************************************************************************
** STATIC pbCallback(<nMessage>) --> <nResult>
**
** This is a sample PVB callback function using psuedo-window output.
**
** Parameters:
**		<nMessage>		a "message" about the status of the backup
**
** Returns:
**		pbCallback() returns a number signifying the action PVB should take.
** 	If the backup should continue with the default action, BACT_OK is
**		returned.  If the backup should be cancelled, BACT_CANCEL is returned.
**		Errors ERROR_CONTAINSDATA and ERROR_WRITE and errors dealing with
**		opening and reading the file have a special return value, BACT_REDO.
**
** With ERROR_CONTAINSDATA, a return value of BACT_REDO means to overwrite
** the disk.  A synonym is BACT_OVERWRITE.  If BACT_OK is returned, PVB
** assumes the user has inserted a new disk and wished to try again.
**
** With ERROR_WRITE, BACT_REDO means to "rewind" the backup to the beginning
** of the current disk.  A synonym is BACT_SKIPVOL.  PVB assumes the user
** has inserted a new disk.  If BACT_OK is returned, PVB retries the write
** operation.
**
** With errors dealing with opening and reading the file, a return value of
** BACT_REDO means to skip the rest of the file and continue with the next
** file.  A synonym is BACT_SKIPFILE.  If BACT_OK is returned, PVB attempts
** to retry or continue with the same file.
**
******************************************************************************
STATIC FUNCTION pbCallback( nMessage )

	STATIC lMediaError := .F.

	LOCAL cColor

	LOCAL nDiskType, ;
			nKey


	IF INKEY() == K_ESC
		TONE(300, 3)
		TONE(100, 3)
		RETURN(BACT_CANCEL)
	ENDIF

	DO CASE
	CASE nMessage == BOP_BEGIN

	CASE nMessage == BOP_CANCEL
		PrintLine("Backup cancelled.")
		RETURN(BACT_CANCEL)

	CASE nMessage == BOP_COMPLETE
		PrintLine("Backup is complete.")

	CASE nMessage == BOP_END

	CASE nMessage == BOP_ERROR
		IF INT(pbError() / 100) == 3

			* test for errors in the "300" range; these normally require
			* insertion of a new disk
			DO CASE
			CASE pbError() == ERROR_CONTAINSDATA
				PrintLine("This disk contains data.  Existing data will be overwritten")
				PrintLine("if you continue.  Do you want to overwrite this disk? ")
				nKey = 0
				DO WHILE (nKey != K_ESC) .AND. ;
						(UPPER(CHR(nKey)) != "Y") .AND. ;
						(UPPER(CHR(nKey)) != "N")
					nKey = INKEY(0)
				ENDDO
				IF nKey == K_ESC
					RETURN(BACT_CANCEL)
				ENDIF
				?? UPPER(CHR(nKey))
				IF UPPER(CHR(nKey)) == "Y"
					RETURN(BACT_OVERWRITE)
				ENDIF

			CASE pbError() == ERROR_DRIVENOTREADY
				PrintLine("Drive not ready.")

			CASE pbError() == ERROR_MEDIATYPE
				// This code attempts to take care of FORMATTED lower-density
				// disks inserted in high-density drives.
				IF .NOT. lMediaError
					nDiskType = pbGetDiskType( pbDriveNum() )
					DO CASE
					CASE nDiskType == DTYPE_1_2
						pbSetDiskType( pbDriveNum(), DTYPE_360 )

					CASE nDiskType == DTYPE_1_44
						pbSetDiskType( pbDriveNum(), DTYPE_720 )

					CASE nDiskType == DTYPE_2_88
						pbSetDiskType( pbDriveNum(), DTYPE_1_44 )

					OTHERWISE
						lMediaError = .T.

					END CASE
				ENDIF

				IF lMediaError
					// If the media type error still occurs after trying all
					// the possible densities for this drive, display a message.
					PrintLine("This disk does not match the media type specified")
					PrintLine("for the backup.  You must now insert a new disk.")
				ELSE
					pb_Reinit( pbDriveNum() )
					RETURN( BACT_OK )
				ENDIF

			CASE pbError() == ERROR_SAMESET
				PrintLine("This is disk " + LTRIM(STR(pbVolNum(), 3, 0)) + ;
					" from the same backup set.")

			CASE pbError() == ERROR_SYSTEMAREA
				PrintLine("The system area of this disk is not usable.  This indicates")
				PrintLine("that the disk is bad or is the wrong media type.  You must")
				PrintLine("now insert a new disk.")

			CASE pbError() == ERROR_TIMEOUT
				PrintLine("Timed out waiting for a disk.")

			CASE pbError() == ERROR_WRITE
				PrintLine("An bad sector was encountered while writing to this disk.")
				PrintLine("The disk is not usable by ProVision:Backup.  You must now")
				PrintLine("insert a new disk.")

			CASE pbError() == ERROR_WRITEPROTECT
				PrintLine("This disk is write-protected.")

			ENDCASE

			PrintLine("Insert disk " + LTRIM(STR(pbWantVolNum(), 3, 0)) + ;
				" in drive " + CHR(pbDriveNum() + 65) + ;
				":, and press a key...")
			IF INKEY(0) == K_ESC
				RETURN(BACT_CANCEL)
			ELSEIF pbError() == ERROR_WRITE
				RETURN(BACT_SKIPVOL)
			ENDIF

		ELSE

			DO CASE
			CASE pbError() == ERROR_FILENOTFOUND
				PrintLine("File not found (" + pbFilename() + ").")

			CASE pbError() == ERROR_FILEOPEN
				PrintLine("Error opening " + pbFilename() + ".")

			CASE pbError() == ERROR_READ
				PrintLine("Error reading " + pbFilename() + ".")

			OTHERWISE
				PrintLine(pb_TransError( pbError(), pbDOSError() ) + ".")

			ENDCASE

			PrintLine("Do you want to skip to the next file? ")
			nKey = 0
			DO WHILE (nKey != K_ESC) .AND. ;
					(UPPER(CHR(nKey)) != "Y") .AND. ;
					(UPPER(CHR(nKey)) != "N")
				nKey = INKEY(0)
			ENDDO
			IF nKey == K_ESC
				RETURN(BACT_CANCEL)
			ENDIF
			?? UPPER(CHR(nKey))
			IF UPPER(CHR(nKey)) == "Y"
				RETURN(BACT_SKIPFILE)
			ENDIF

		ENDIF

	CASE nMessage == BOP_FILEBEGIN
		PrintLine(pbFilename() + " (CTYPE=" + STR(pbCmpType(), 1, 0) + ")")

	CASE nMessage == BOP_FILEEND

	CASE nMessage == BOP_FILEESTIMATE
		PrintLine("Estimating " + pbFilename())

	CASE nMessage == BOP_NEXTVOL
		PrintLine("Insert disk " + LTRIM(STR(pbWantVolNum(), 2, 0)) + ;
			" in drive " + CHR(pbDriveNum() + 65) + ":")
		IF pbGetDetectChg()
			?? "..."
		ELSE
			?? ", and press a key..."
			IF INKEY(0) == K_ESC
				RETURN(BACT_CANCEL)
			ENDIF
		ENDIF

	CASE nMessage == BOP_RESETARCHIVE
		PrintLine("Resetting archive bits...")

	CASE nMessage == BOP_TRACKFORMATTED
		IF ISCOLOR()
			cColor = SETCOLOR("B/W")
		ELSE
			cColor = SETCOLOR("N/W")
		ENDIF
		@ 24,00 CLEAR TO 24,79
		@ 24,24 SAY "Formatting and writing cylinder " + ;
			STRZERO(pbCylNum(), 2) + " head " + STRZERO(pbHeadNum(), 2) + ;
			"  " + STR((pbFileRead() * 100) / pbFileSize(), 3, 0) + "%"
		SETCOLOR(cColor)

	CASE nMessage == 7
		IF ISCOLOR()
			cColor = SETCOLOR("B/W")
		ELSE
			cColor = SETCOLOR("N/W")
		ENDIF
		@ 24,00 CLEAR TO 24,79
		@ 24,24 SAY "Writing cylinder " + STRZERO(pbCylNum(), 2) + ;
			" head " + STRZERO(pbHeadNum(), 2) + "  " + ;
			STR((pbFileRead() * 100) / pbFileSize(), 3, 0) + "%"
		SETCOLOR(cColor)

	ENDCASE

	RETURN(BACT_OK)



******************************************************************************
** PrintLine(<cString>) --> NIL
**
** This function prints a line in the psuedo-window.
**
** Parameters:
**		<cString>		a "string" to display in the window
**
** Returns:
**		NIL
**
******************************************************************************
FUNCTION PrintLine(cString)


	// Scroll psuedo-window
	SCROLL(05, 10, 20, 69, 1)

	// Write the string
	@ 20,10 SAY SUBSTR(cString, 1, 60)

	RETURN(NIL)

