/*****************************************************************************
** PBREST.PRG
**
** ProVision:Backup
** CA-Clipper 5.0 surface restore sample
**
** Copyright 1993 SofDesign International, Inc.
** All rights reserved
**
** tab spacing = 3
**
*****************************************************************************/

#include "inkey.ch"
#include "pb.ch"


// direct output to stderr
#xcommand	? <exp> ;
				=> ;
				OUTERR( CRLF + <exp> )

#xcommand	?? <exp> ;
				=> ;
				OUTERR( <exp> )

#define CRLF	CHR(13) + CHR(10)			// carriage return and line feed


STATIC	lCancelled	:= .F.



/*****************************************************************************
** Main( <cFileSpec> [, <cDrive> ] [, <cDest> ] ) --> NIL
**
** This is the main module of a simple PVB program using TTY-type output.
**
** Parameters:
**		<cFileSpec>		a filespec to be restored
**
**		<cDrive>			the source drive (A: or B:); may contain a format
**								switch of /360, /1.2, /720, /1.44, or /2.88, but if
**								so, there cannot be any spaces between the drive
**								and the switch
**
**		<cDest>			the destination drive
**
** Returns:
**		NIL
**
** The limitation of no spaces before switches applies to this sample
** program only, because it only retrieves the first two parameters from
** the command line.
**
*****************************************************************************/
FUNCTION Main( cFileSpec, cDrive, cDest )


	// validate command line params
	IF ( EMPTY( cFileSpec ) )
		? "Usage:  PBREST <filespec> [<drive>[/360|/1.2|/720|/1.44|/2.88]] [<dest>]"
		?
		? "Restores the specified filespec from the specified drive.  There can"
		? "be no spaces between drive and format."
		?
		? "<drive> defaults to A:"
		?
		? "<dest> defaults to nothing"
		?
		RETURN( NIL )
	END IF // ( EMPTY( cFileSpec ) )

	IF ( EMPTY( cDrive ) )
		cDrive	:= "A"
	END IF // ( EMPTY( cDrive ) )

	// specify the disk type?
	IF ( "/360" $ cDrive )
		pbSetDiskType( 0, DTYPE_360 )
		pbSetDiskType( 1, DTYPE_360 )
	ELSEIF ( "/1.2" $ cDrive )
		pbSetDiskType( 0, DTYPE_1_2 )
		pbSetDiskType( 1, DTYPE_1_2 )
	ELSEIF ( "/720" $ cDrive )
		pbSetDiskType( 0, DTYPE_720 )
		pbSetDiskType( 1, DTYPE_720 )
	ELSEIF ( "/1.44" $ cDrive )
		pbSetDiskType( 0, DTYPE_1_44 )
		pbSetDiskType( 1, DTYPE_1_44 )
	ELSEIF ( "/2.88" $ cDrive )
		pbSetDiskType( 0, DTYPE_2_88 )
		pbSetDiskType( 1, DTYPE_2_88 )
	END IF // ( "/360" $ cDrive )

	// perform the restore
	IF ( .NOT. pbRestore( { cFileSpec }, {}, cDrive, { | nMessage | ;
			pbCallback( nMessage ) }, cDest ) )

		IF ( .NOT. lCancelled )
			// display error message
			? CHR( 7 ) + "Error=" + STRZERO( pbError(), 4 )
			IF ( pbError() == ERROR_DOSERROR )
				?? " (DOS error=" + STR( pbDOSError(), 3, 0 ) + ")"
			ELSEIF ( pbError() == ERROR_BIOSERROR )
				?? " (BIOS error=" + STR( pbDOSError(), 3, 0 ) + ")"
			END IF // ( pbError() == ERROR_DOSERROR )
			? pb_TransError( pbError(), pbDOSError() )
			?? CRLF
		END IF // ( .NOT. lCancelled )

	END IF // ( .NOT. pbBackup( { cFileSpec }, {}, cDrive, ...

	RETURN( NIL )
	// END Main( cFileSpec, cDrive )



/*****************************************************************************
** STATIC pbCallback( <nMessage> ) --> <nResult>
**
** This is a sample PVB callback function using TTY-type output.
**
** Parameters:
**		<nMessage>		a "message" about the status of the restore
**
** Returns:
**		pbCallback() returns a number signifying the action PVB should take.
** 	If the restore should continue with the default action, BACT_OK is
**		returned.  If the restore should be cancelled, BACT_CANCEL is returned.
**		BOP_FILEEXISTS and error ERROR_WRITE and errors dealing with
**		opening and reading the file have a special return value, BACT_REDO.
**
** With BOP_FILEEXISTS, a return value of BACT_REDO means to overwrite tje
** file.  A synonym is BACT_OVERWRITE.  If BACT_OK is returned, PVB assumes
** the user wishes to skip the existing file.
**
** With errors dealing with creating and writing the file, a return value of
** BACT_REDO means to skip the rest of the file and continue with the next
** file.  A synonym is BACT_SKIPFILE.  If BACT_OK is returned, PVB attempts
** to retry or continue with the same file.
**
*****************************************************************************/
STATIC FUNCTION pbCallback( nMessage )

	STATIC	lMediaError	:= .F.

	LOCAL	nDiskType, ;
			nKey


	IF ( INKEY() == K_ESC )
		lCancelled	:= .T.
		RETURN( BACT_CANCEL )
	END IF // ( INKEY() == K_ESC )

	DO CASE
	CASE ( nMessage == BOP_BEGIN )

	CASE ( nMessage == BOP_CANCEL )
		? "Restore cancelled." + CRLF
		RETURN( BACT_CANCEL )

	CASE ( nMessage == BOP_COMPLETE )
		? "Restore is complete." + CRLF

	CASE ( nMessage == BOP_END )

	CASE ( nMessage == BOP_ERROR )
		IF ( INT( pbError() / 100 ) == 3 )

			// test for errors in the "300" range
			DO CASE
			CASE ( pbError() == ERROR_DRIVENOTREADY )
				? "Drive not ready."

			CASE ( pbError() == ERROR_MEDIATYPE )
				// This code attempts to take care of FORMATTED lower-density
				// disks inserted in high-density drives.
				IF ( .NOT. lMediaError )
					nDiskType	:= pbGetDiskType( pbDriveNum() )
					DO CASE
					CASE ( nDiskType == DTYPE_1_2 )
						pbSetDiskType( pbDriveNum(), DTYPE_360 )

					CASE ( nDiskType == DTYPE_1_44 )
						pbSetDiskType( pbDriveNum(), DTYPE_720 )

					CASE ( nDiskType == DTYPE_2_88 )
						pbSetDiskType( pbDriveNum(), DTYPE_1_44 )

					OTHERWISE
						lMediaError	:= .T.

					END CASE
				END IF // ( .NOT. lMediaError )

				IF ( lMediaError )
					// If the media type error still occurs after trying all
					// the possible densities for this drive, display a message.
					? "This disk does not match the media type specified for " + ;
						"the restore."
				ELSE
					pb_Reinit( pbDriveNum() )
					RETURN( BACT_OK )
				END IF // ( lMediaError )

			CASE ( pbError() == ERROR_NONPVB )
				? "This is not a ProVision:Backup disk."

			CASE ( pbError() == ERROR_SEQUENCE )
				? "This is disk " + LTRIM( STR( pbVolNum(), 3, 0 ) ) + ;
					" from the backup set."
				? "Do you want to begin with this disk? "
				nKey	:= 0
				WHILE ( ( nKey != K_ESC ) .AND. ;
						( UPPER( CHR( nKey ) ) != "Y" ) .AND. ;
						( UPPER( CHR( nKey ) ) != "N" ) )
					nKey	:= INKEY( 0 )
				END WHILE // ( ( nKey != K_ESC ) .AND. ...
				IF ( nKey == K_ESC )
					lCancelled	:= .T.
					RETURN( BACT_CANCEL )
				END IF // ( nKey == K_ESC )
				?? UPPER( CHR( nKey ) ) + CRLF
				IF ( UPPER( CHR( nKey ) ) == "Y" )
					RETURN( BACT_REDO )
				END IF // ( UPPER( CHR( nKey ) ) == "Y" )

			CASE ( pbError() == ERROR_TIMEOUT )
				? "Timed out waiting for a disk."

			END CASE

			IF ( pbWantVolNum() > 999 )
				? "Insert the last disk"
			ELSE
				? "Insert disk " + LTRIM( STR( pbWantVolNum(), 3, 0 ) )
			END IF // ( pbWantVolNum() > 999 )
			?? " in drive " + CHR( pbDriveNum() + 65 ) + ;
				":, and press a key..." + CRLF
			IF ( INKEY( 0 ) == K_ESC )
				lCancelled	:= .T.
				RETURN( BACT_CANCEL )
			END IF // ( INKEY( 0 ) == K_ESC )

		ELSE

			DO CASE
			CASE ( pbError() == ERROR_FILEOPEN )
				? "Error opening " + pbFilename() + "." + CRLF

			CASE ( pbError() == ERROR_READ )
				? "Error reading " + pbFilename() + "." + CRLF

			CASE ( pbError() == ERROR_CRC )
				? "CRC error restoring " + pbFilename() + "." + CRLF

			OTHERWISE
				? pb_TransError( pbError(), pbDOSError() ) + "." + CRLF

			END CASE

			? "Do you want to skip to the next file? "
			nKey	:= 0
			WHILE ( ( nKey != K_ESC ) .AND. ;
					( UPPER( CHR( nKey ) ) != "Y" ) .AND. ;
					( UPPER( CHR( nKey ) ) != "N" ) )
				nKey	:= INKEY( 0 )
			END WHILE // ( ( nKey != K_ESC ) .AND. ...
			IF ( nKey == K_ESC )
				lCancelled	:= .T.
				RETURN( BACT_CANCEL )
			END IF // ( nKey == K_ESC )
			?? UPPER( CHR( nKey ) ) + CRLF
			IF ( UPPER( CHR( nKey ) ) == "Y" )
				RETURN( BACT_SKIPFILE )
			END IF // ( UPPER( CHR( nKey ) ) == "Y" )

		END IF // ( INT( pbError() / 100 ) == 3 )

	CASE ( nMessage == BOP_FILEBEGIN )
		?? CHR( 13 ) + PADR( pbFilename() + " (CTYPE=" + ;
			STR( pbCmpType(), 1, 0 ) + ")", 79 ) + CRLF

	CASE ( nMessage == BOP_FILEEND )

	CASE ( nMessage == BOP_FILEEXISTS )
		? pbFilename() + " exists."
		? "The existing file will be overwritten if you continue."
		? "Do you want to overwrite this file? "
		nKey	:= 0
		WHILE ( ( nKey != K_ESC ) .AND. ;
				( UPPER( CHR( nKey ) ) != "Y" ) .AND. ;
				( UPPER( CHR( nKey ) ) != "N" ) )
			nKey	:= INKEY( 0 )
		END WHILE // ( ( nKey != K_ESC ) .AND. ...
		IF ( nKey == K_ESC )
			lCancelled	:= .T.
			RETURN( BACT_CANCEL )
		END IF // ( nKey == K_ESC )
		?? UPPER( CHR( nKey ) ) + CRLF
		IF ( UPPER( CHR( nKey ) ) == "Y" )
			RETURN( BACT_OVERWRITE )
		END IF // ( UPPER( CHR( nKey ) ) == "Y" )

	CASE ( nMessage == BOP_NEXTVOL )
		IF ( pbWantVolNum() > 999 )
			? "Insert the last disk"
		ELSE
			? "Insert disk " + LTRIM( STR( pbWantVolNum(), 2, 0 ) )
		END IF // ( pbWantVolNum() > 999 )
		?? " in drive " + CHR( pbDriveNum() + 65 ) + ":"
		IF ( pbGetDetectChg() )
			?? "..." + CRLF
		ELSE
			?? ", and press a key..." + CRLF
			IF ( INKEY( 0 ) == K_ESC )
				lCancelled	:= .T.
				RETURN( BACT_CANCEL )
			END IF // ( INKEY( 0 ) == K_ESC )
		END IF // ( pbGetDetectChg() )

	CASE ( nMessage == BOP_TRACKREAD )
		?? CHR( 13 ) + "Reading cylinder " + STRZERO( pbCylNum(), 2 ) + ;
			" head " + STRZERO( pbHeadNum(), 2 ) + "  " + ;
			STR( ( pbFileRead() * 100 ) / pbFileSize(), 3, 0 ) + "%"

	END CASE

	RETURN( BACT_OK )
	// END pbCallback( nMessage )

