******************************************************************************
** PBREST2.PRG
**
** ProVision:Backup
** CA-Clipper 5.0 surface interactive restore sample
**
** Copyright 1993 SofDesign International, Inc.
** All rights reserved
**
** tab spacing = 3
**
******************************************************************************

#include "inkey.ch"
#include "pb.ch"



/*****************************************************************************
** Main() --> NIL
**
** This is the main module of a simple interactive PVB program.
**
** Returns:
**		NIL
**
*****************************************************************************/
FUNCTION Main()

	LOCAL GetList := {}

	// GET variables
	LOCAL acInclude[3]
	LOCAL alSubDir[3]
	LOCAL acExclude[3]
	LOCAL cDrive, nDriveType, nOverwrite, lAutoDetect, cPassword

	// Other variables
	LOCAL i, cColor


	// Clear the screen and paint the input screen
	IF ISCOLOR()
		SET COLOR TO +W/B, B/W
	ELSE
		SET COLOR TO W/N, N/W
	ENDIF
	CLEAR SCREEN
	@ 01,06 SAY "ProVision:Backup (Sample Restore) by SofDesign International, Inc."
	@ 03,02 SAY ""+replicate('',30)+""+replicate('',8)+"   "+replicate('',30)+""
	@ 04,02 SAY "Restore Filespecs:"+space(12)+"SubDirs?   Exclude Filespecs:"+space(12)+""
	@ 05,02 SAY ""+replicate('',30)+""+replicate('',8)+"   "+replicate('',30)+""
	@ 06,02 SAY ""+space(30)+"           "+space(30)+""
	@ 07,02 SAY ""+space(30)+"           "+space(30)+""
	@ 08,02 SAY ""+space(30)+"           "+space(30)+""
	@ 09,02 SAY ""+replicate('',30)+""+replicate('',8)+"   "+replicate('',30)+""
	@ 11,21 SAY ""+replicate('',36)+""
	@ 12,21 SAY "     Restore from drive (A/B):"+space(6)+""
	@ 13,21 SAY "     Restore drive capacity..:"+space(6)+""
	@ 14,21 SAY ""+space(7)+"1) 360K,  2) 1.2M"+space(12)+""
	@ 15,21 SAY ""+space(7)+"3) 720K,  4) 1.44M"+space(11)+""
	@ 16,21 SAY ""+space(7)+"5) 2.88M  6) Auto"+space(12)+""
	@ 17,21 SAY "     Overwrite Warnings......:"+space(6)+""
	@ 18,21 SAY ""+space(7)+"1) Never warn"+space(16)+""
	@ 19,21 SAY ""+space(7)+"2) Always warn"+space(15)+""
	@ 20,21 SAY "     Detect diskette change?.:"+space(6)+""
	@ 21,21 SAY "     Password:"+space(22)+""
	@ 22,21 SAY ""+replicate('',36)+""

	// Initialize the input variables
	AFILL(acInclude, SPACE(30))
	AFILL(alSubDir, .F.)
	AFILL(acExclude, SPACE(30))
	cDrive			:= "A"
	nDriveType		:= 6
	nOverwrite		:= 2
	lAutoDetect		:= .T.
	cPassword		:= SPACE(14)

	// Do the GETs
	@ 06,03 GET acInclude[1]  PICTURE "@!"
	@ 06,38 GET alSubDir[1]   PICTURE "Y"
	@ 07,03 GET acInclude[2]  PICTURE "@!"
	@ 07,38 GET alSubDir[2]   PICTURE "Y"
	@ 08,03 GET acInclude[3]  PICTURE "@!"
	@ 08,38 GET alSubDir[3]   PICTURE "Y"
	@ 06,47 GET acExclude[1]  PICTURE "@!"
	@ 07,47 GET acExclude[2]  PICTURE "@!"
	@ 08,47 GET acExclude[3]  PICTURE "@!"
	@ 12,52 GET cDrive        PICTURE "!" VALID cDrive $ "AB"
	@ 13,52 GET nDriveType    PICTURE "9" RANGE 1,6
	@ 17,52 GET nOverwrite    PICTURE "9" RANGE 1,2
	@ 20,52 GET lAutoDetect   PICTURE "Y"
	@ 21,37 GET cPassword     PICTURE "@!"

	// Get input and return if escape is pressed
	READ
	IF LASTKEY() == K_ESC
		RETURN(NIL)
	ENDIF

	// Test for include subdirs
	FOR i := 1 TO 3
		acInclude[i] := LTRIM( RTRIM( acInclude[i] ) ) + ;
			IIF( alSubDir[i], "/S", "" )
		acExclude[i] := LTRIM( RTRIM( acExclude[i] ) )
	NEXT i 

	// Set the various options
	IF .NOT. EMPTY(cPassword)
		pbSetPassword(ALLTRIM(cPassword))
	ENDIF
	pbSetDetectChg(lAutoDetect)
	pbSetDiskType(ASC(cDrive) - ASC("A"), nDriveType - 1 )
	pbSetOverwriteOpt(IIF(nOverwrite == 1, WARN_NONE, WARN_ALL))

	// Create a psuedo-window
	IF ISCOLOR()
		SET COLOR TO +W/R
	ELSE
		SET COLOR TO N/W
	ENDIF
	@ 04,09 TO 21,70
	@ 05,10 CLEAR TO 20,69
	@ 20,10 SAY ""


	// Perform the restore
	IF !pbRestore(acInclude, acExclude, cDrive, ;
			{|nMessage| pbCallback(nMessage)})

		// Display error message
		IF pbError() != ERROR_NONE
			TONE(100, 3)
			PrintLine("Error=" + STRZERO(pbError(), 4))
			IF pbError() == 205
				PrintLine(" (DOS error=" + STR(pbDOSError(), 3, 0) + ")")
			ELSEIF pbError() == 206
				PrintLine(" (BIOS error=" + STR(pbDOSError(), 3, 0) + ")")
			ENDIF
			PrintLine("")
		ENDIF

	ENDIF


	// Clear status line at bottom
	cColor := SETCOLOR()
	IF ISCOLOR()
		SET COLOR TO +W/B, B/W
	ELSE
		SET COLOR TO W/N, N/W
	ENDIF
	@ 24,00 CLEAR TO 24,79
	SETCOLOR(cColor)

	// Ask user to press a key
	PrintLine("Press any key to return to DOS...")

	// Wait for keystroke
	CLEAR TYPEAHEAD
	INKEY(0)

	// Clear screen and return
	SET COLOR TO W/N
	CLEAR SCREEN

	RETURN(NIL)



******************************************************************************
** STATIC pbCallback(<nMessage>) --> <nResult>
**
** This is a sample PVB callback function using psuedo-window output.
**
** Parameters:
**		<nMessage>		a "message" about the status of the backup
**
** Returns:
**		pbCallback() returns a number signifying the action PVB should take.
** 	If the restore should continue with the default action, BACT_OK is
**		returned.  If the restore should be cancelled, BACT_CANCEL is returned.
**		BOP_FILEEXISTS and error ERROR_WRITE and errors dealing with
**		opening and reading the file have a special return value, BACT_REDO.
**
** With BOP_FILEEXISTS, a return value of BACT_REDO means to overwrite tje
** file.  A synonym is BACT_OVERWRITE.  If BACT_OK is returned, PVB assumes
** the user wishes to skip the existing file.
**
** With errors dealing with creating and writing the file, a return value of
** BACT_REDO means to skip the rest of the file and continue with the next
** file.  A synonym is BACT_SKIPFILE.  If BACT_OK is returned, PVB attempts
** to retry or continue with the same file.
**
******************************************************************************
STATIC FUNCTION pbCallback(nMessage)

	STATIC lMediaError := .F.

	LOCAL cColor

	LOCAL nDiskType, ;
			nKey


	IF INKEY() == K_ESC
		TONE(300, 3)
		TONE(100, 3)
		RETURN(BACT_CANCEL)
	ENDIF

	DO CASE
	CASE nMessage == BOP_BEGIN

	CASE nMessage == BOP_CANCEL
		PrintLine("Restore cancelled.")
		RETURN(-1)

	CASE nMessage == BOP_COMPLETE
		PrintLine("Restore is complete.")

	CASE nMessage == BOP_END

	CASE nMessage == BOP_ERROR
		IF INT(pbError() / 100) == 3

			* test for errors in the "300" range
			DO CASE
			CASE pbError() == ERROR_DRIVENOTREADY
				PrintLine("Drive not ready.")

			CASE pbError() == ERROR_MEDIATYPE
				// This code attempts to take care of FORMATTED lower-density
				// disks inserted in high-density drives.
				IF .NOT. lMediaError
					nDiskType = pbGetDiskType(pbDriveNum())
					DO CASE
					CASE nDiskType == DTYPE_1_2
						pbSetDiskType(pbDriveNum(), DTYPE_360)

					CASE nDiskType == DTYPE_1_44
						pbSetDiskType(pbDriveNum(), DTYPE_720)

					CASE nDiskType == DTYPE_2_88
						pbSetDiskType(pbDriveNum(), DTYPE_1_44)

					OTHERWISE
						lMediaError = .T.

					ENDCASE
				ENDIF

				IF lMediaError
					// If the media type error still occurs after trying all
					// the possible densities for this drive, display a message.
					PrintLine("This disk does not match the media type")
					PrintLine("specified for the restore.")
				ELSE
					pb_Reinit(pbDriveNum())
					RETURN(BACT_OK)
				ENDIF

			CASE pbError() == ERROR_NONPVB
				PrintLine("This is not a ProVision:Backup disk.")

			CASE pbError() == ERROR_SEQUENCE
				PrintLine("This is disk " + LTRIM(STR(pbVolNum(), 3, 0)) + ;
					" from the backup set.")
				PrintLine("Do you want to begin with this disk? ")
				nKey	= 0
				DO WHILE (nKey != 27) .AND. ;
						(UPPER(CHR(nKey)) != "Y") .AND. ;
						(UPPER(CHR(nKey)) != "N")
					nKey = INKEY(0)
				ENDDO
				IF nKey == K_ESC
					RETURN(BACT_CANCEL)
				ENDIF
				?? UPPER(CHR(nKey))
				IF UPPER(CHR(nKey)) == "Y"
					RETURN(BACT_REDO)
				ENDIF

			CASE pbError() == ERROR_TIMEOUT
				PrintLine("Timed out waiting for a disk.")

			ENDCASE

			IF pbWantVolNum() > 999
				PrintLine("Insert the last disk in drive " + ;
					CHR(pbDriveNum() + 65) + ":, and press a key...")
			ELSE
				PrintLine("Insert disk " + LTRIM(STR(pbWantVolNum(), 3, 0)) + ;
					" in drive " + CHR(pbDriveNum() + 65) + ;
					":, and press a key...")
			ENDIF
			IF INKEY(0) == K_ESC
				RETURN(BACT_CANCEL)
			ENDIF

		ELSE

			DO CASE
			CASE pbError() == ERROR_FILENOTFOUND
				PrintLine("File not found (" + pbFilename() + ").")

			CASE pbError() == ERROR_FILEOPEN
				PrintLine("Error opening " + pbFilename() + ".")

			CASE pbError() == ERROR_READ
				PrintLine("Error reading " + pbFilename() + ".")

			OTHERWISE
				PrintLine(pb_TransError( pbError(), pbDOSError() ) + ".")

			ENDCASE

			PrintLine("Do you want to skip to the next file? ")
			nKey = 0
			DO WHILE (nKey != 27) .AND. ;
					(UPPER(CHR(nKey)) != "Y") .AND. ;
					(UPPER(CHR(nKey)) != "N")
				nKey = INKEY(0)
			ENDDO
			IF nKey == K_ESC
				RETURN(BACT_CANCEL)
			ENDIF
			?? UPPER(CHR(nKey))
			IF UPPER(CHR(nKey)) == "Y"
				RETURN(BACT_SKIPFILE)
			ENDIF

		ENDIF

	CASE nMessage == BOP_FILEBEGIN
		PrintLine(pbFilename() + " (CTYPE=" + STR(pbCmpType(), 1, 0) + ")")

	CASE nMessage == BOP_FILEEND

	CASE nMessage == BOP_FILEEXISTS
		PrintLine(pbFilename() + " exists.")
		PrintLine("The existing file will be " + ;
			"overwritten if you continue.")
		PrintLine("Do you want to overwrite this file? ")
		nKey = 0
		DO WHILE (nKey != 27) .AND. ;
				(UPPER(CHR(nKey)) != "Y") .AND. ;
				(UPPER(CHR(nKey)) != "N")
			nKey = INKEY(0)
		ENDDO
		IF nKey == K_ESC
			RETURN(BACT_CANCEL)
		ENDIF
		?? UPPER(CHR(nKey))
		IF UPPER(CHR(nKey)) == "Y"
			RETURN(BACT_OVERWRITE)
		ENDIF

	CASE nMessage == BOP_NEXTVOL
		IF pbWantVolNum() > 999
			PrintLine("Insert the last disk in drive " + ;
				CHR(pbDriveNum() + 65) + ":, and press a key...")
		ELSE
			PrintLine("Insert disk " + LTRIM(STR(pbWantVolNum(), 2, 0)) + ;
				" in drive " + CHR(pbDriveNum() + 65) + ":")
		ENDIF
		IF pbGetDetectChg()
			?? "..."
		ELSE
			?? ", and press a key..."
			IF INKEY(0) == K_ESC
				RETURN(BACT_CANCEL)
			ENDIF
		ENDIF

	ENDCASE

	RETURN(BACT_OK)



******************************************************************************
** PrintLine(<cString>) --> NIL
**
** This function prints a line in the psuedo-window.
**
** Parameters:
**		<cString>		a "string" to display in the window
**
** Returns:
**		NIL
**
******************************************************************************
FUNCTION PrintLine(cString)


	// Scroll psuedo-window
	SCROLL(05, 10, 20, 69, 1)

	// Write the string
	@ 20,10 SAY SUBSTR(cString, 1, 60)

	RETURN(NIL)



